require "import"
import "android.content.Context"
import "android.os.Vibrator"
import "android.media.ToneGenerator"
import "android.media.AudioManager"
import "android.widget.*"
import "com.androlua.*"
import "android.net.Uri"
import "android.content.Intent"
import "android.text.Html"

local context = activity or service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
vibrator.vibrate(40)

local translationResult = ""
local languageCodes = {
    ["English"] = "en",
    ["Esperanto"] = "eo",
    ["Estonian"] = "et",
    ["Ewe"] = "ee",
    ["Faroese"] = "fo",
    ["Filipino"] = "tl",
    ["Finnish"] = "fi",
    ["French"] = "fr",
    ["Frisian"] = "fy",
    ["Ga"] = "gaa",
    ["Galician"] = "gl",
    ["Georgian"] = "ka",
    ["German"] = "de",
    ["Greek"] = "el",
    ["Guarani"] = "gn",
    ["Gujarati"] = "gu",
    ["Hacker"] = "xx-hacker",
    ["Haitian Creole"] = "ht",
    ["Hausa"] = "ha",
    ["Hawaiian"] = "haw",
    ["Hebrew"] = "iw",
    ["Hindi"] = "hi",
    ["Hungarian"] = "hu",
    ["Icelandic"] = "is",
    ["Igbo"] = "ig",
    ["Indonesian"] = "id",
    ["Interlingua"] = "ia",
    ["Irish"] = "ga",
    ["Italian"] = "it",
    ["Japanese"] = "ja",
    ["Javanese"] = "jw",
    ["Kannada"] = "kn",
    ["Kazakh"] = "kk",
    ["Kinyarwanda"] = "rw",
    ["Kirundi"] = "rn",
    ["Klingon"] = "xx-klingon",
    ["Kongo"] = "kg",
    ["Korean"] = "ko",
    ["Krio (Sierra Leone)"] = "kri",
    ["Kurdish"] = "ku",
    ["Kurdish (Soranî)"] = "ckb",
    ["Kyrgyz"] = "ky",
    ["Laothian"] = "lo",
    ["Latin"] = "la",
    ["Latvian"] = "lv",
    ["Lingala"] = "ln",
    ["Lithuanian"] = "lt",
    ["Lozi"] = "loz",
    ["Luganda"] = "lg",
    ["Luo"] = "ach",
    ["Macedonian"] = "mk",
    ["Malagasy"] = "mg",
    ["Malay"] = "ms",
    ["Malayalam"] = "ml",
    ["Divehi"] = "mt",
    ["Maltese"] = "mt",
    ["Maori"] = "mi",
    ["Marathi"] = "mr",
    ["Mauritian Creole"] = "mfe",
    ["Moldavian"] = "mo",
    ["Mongolian"] = "mn",
    ["Montenegrin"] = "sr-ME",
    ["Nepali"] = "ne",
    ["Nigerian Pidgin"] = "pcm",
    ["Northern Sotho"] = "nso",
    ["Norwegian"] = "no",
    ["Norwegian (Nynorsk)"] = "nn",
    ["Occitan"] = "oc",
    ["Oriya"] = "or",
    ["Oromo"] = "om",
    ["Pashto"] = "ps",
    ["Persian"] = "fa",
    ["Pirate"] = "xx-pirate",
    ["Polish"] = "pl",
    ["Portuguese (Brazil)"] = "pt-BR",
    ["Portuguese (Portugal)"] = "pt-PT",
    ["Punjabi"] = "pa",
    ["Quechua"] = "qu",
    ["Romanian"] = "ro",
    ["Romansh"] = "rm",
    ["Runyakitara"] = "nyn",
    ["Russian"] = "ru",
    ["Scots Gaelic"] = "gd",
    ["Serbian"] = "sr",
    ["Serbo-Croatian"] = "sh",
    ["Sesotho"] = "st",
    ["Setswana"] = "tn",
    ["Seychellois Creole"] = "crs",
    ["Shona"] = "sn",
    ["Sindhi"] = "sd",
    ["Sinhalese"] = "si",
    ["Slovak"] = "sk",
    ["Slovenian"] = "sl",
    ["Somali"] = "so",
    ["Spanish"] = "es",
    ["Spanish (Latin American)"] = "es-419",
    ["Sundanese"] = "su",
    ["Swahili"] = "sw",
    ["Swedish"] = "sv",
    ["Tajik"] = "tg",
    ["Tamil"] = "ta",
    ["Tatar"] = "tt",
    ["Telugu"] = "te",
    ["Thai"] = "th",
    ["Tigrinya"] = "ti",
    ["Tonga"] = "to",
    ["Tshiluba"] = "lua",
    ["Tumbuka"] = "tum",
    ["Turkish"] = "tr",
    ["Turkmen"] = "tk",
    ["Twi"] = "tw",
    ["Uighur"] = "ug",
    ["Ukrainian"] = "uk",
    ["Urdu"] = "ur",
    ["Uzbek"] = "uz",
    ["Vietnamese"] = "vi",
    ["Welsh"] = "cy",
    ["Wolof"] = "wo",
    ["Xhosa"] = "xh",
    ["Yiddish"] = "yi",
    ["Yoruba"] = "yo",
    ["Zulu"] = "zu",
}

local items = {}
for language, code in pairs(languageCodes) do
    table.insert(items, language)
end
table.sort(items)

function capitalizeFirstWord(str)
    return str:gsub("^%l", string.upper)
end

function addPunctuation(str)
    return str .. "."
end

function hasInternetConnection()
    local connectivityManager = context.getSystemService(Context.CONNECTIVITY_SERVICE)
    local networkInfo = connectivityManager.getActiveNetworkInfo()
    return networkInfo ~= nil and networkInfo.isConnected()
end

if not hasInternetConnection() then
    service.asyncSpeak("No internet connection. Unable to translate language")
    return false
end

local layout = {
    LinearLayout;
    layout_width = "fill";
    layout_height = "fill";
    orientation = "vertical";
    {
        EditText,
        id = "searchBox";
        layout_width = "fill";
        layout_height = "wrap";
        hint = "Find the language you want to translate to...";
    };
    {
        GridView,
        id = "grid";
        numColumns = 1;
        layout_width = "fill";
        layout_height = "0dp";
        layout_weight = 1;
    },
    {
        Button,
        id = "exitButton";
        text = "Exit";
        layout_width = "fill";
        layout_height = "wrap";
    }
}

local dlg = LuaDialog(context)
dlg.View = loadlayout(layout)

local adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, items)
grid.adapter = adapter

dlg.setTitle("Google Translate")
    .setMessage("Welcome to Google Translate! One of the jieshuo extensions to make your life easier!")
    .show()

searchBox.addTextChangedListener{
    onTextChanged = function(s, start, before, count)
        local keyword = s.toString()
        local filteredItems = {}
        for _, language in ipairs(items) do
            if language:lower():find(keyword:lower()) then
                table.insert(filteredItems, language)
            end
        end
        adapter.clear()
        adapter.addAll(filteredItems)
        adapter.notifyDataSetChanged()
    end
}
grid.onItemClick = function(l, v)
    dlg.dismiss()
    task(500, function()
        local language = v.text
        local defaultLanguage = Locale.getDefault().getLanguage()
        local languageCode = languageCodes[language]

        if languageCode then
            local url = "https://translate.google.com/m?sl=auto&hl=" .. defaultLanguage .. "&q=" .. Uri.encode(context.getText(node))
            url = url .. "&tl=" .. languageCode

            Http.get(url, function(status, result)
                if status == 200 then
                    local translation = tostring(Html.fromHtml(result:match('<div class="result%-container">([^<]+)</div>')))
                    translation = capitalizeFirstWord(translation)
                    translation = addPunctuation(translation)
                    translationResult = translationResult .. translation
                    service.setText(translationResult)
                    service.speak("Translating text: " .. translationResult)
                    vibrator.vibrate(100)
                else
                    print("Sorry, an error occurred.\n These methods may be helpful:\n - Ensure you are connected to the internet.\n - Try to provide a stable internet connection.\n Note: Some elements, images, text boxes, etc., may not be translated.")
                    vibrator.vibrate(200)
                    service.playSoundTick()
                end
            end)
        else
            print("Language not supported")
            vibrator.vibrate(200)
            service.playSoundClock()
        end
    end)
end

exitButton.onClick = function()
    service.playSoundTick()
    service.speak("Thank you for using this extension, translated and modified by: Muhammad Gagah.")
    dlg.dismiss()
end
